/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pControler;
import java.util.HashMap;
import java.util.Map;

import pModel.Match;
import pModel.Solo;
import pModel._Model;
import pVue.VueControle;
import pVue.VueJeuEquipe;
import pVue.VueJeuSolo;
import pVue._Vue;


/**
 * Class ControlerJeu :
 * Controleur de l'application
 * @author Olivier Berthier
 */
public class ControlerJeu {

	//
	// Fields
	//
	/**
	 * Interface de contrôle
	 */
	private VueControle vue;
	/**
	 * HashMap permettant d'associer des Vues et des Models.
	 * 
	 * @see map
	 * @see pModel._Model
	 * @see pVue._Vue
	 */
	private static Map<_Model,_Vue> modelsVues= new HashMap<_Model,_Vue>();
	//
	// Constructors
	//
	/**
	 * Constructeur par défaut
	 */
	public ControlerJeu () {
		this.initGUI();
	};

	//
	// Methods
	//
	/**
	 * Initialiser l'interface de contrôle uniquement.
	 */
	public void initGUI(){
		this.vue = new VueControle(this);

	}

	/**
	 * Creer un tortue Solo
	 * 
	 * @see pModel.Solo#Solo()
	 * @see pVue.VueJeuSolo#VueJeuSolo(pMode._Model)
	 */
	public void initSolo(){
		Solo s = new Solo();
		VueJeuSolo v = new VueJeuSolo(s);
		this.modelsVues.put(s, v);
	}

	/**
	 * Créer un match entre plusieurs équipes
	 * 
	 * @see pModel.Match#Match()
	 * @see pVue.VueJeuEquipe#VueJeuEquipe(pModel._Model)
	 */
	public Match initMatch(){
		//Initialisation
		Match m = new Match();
		VueJeuEquipe v = new VueJeuEquipe(m);
		m.addObserver(v);
		this.modelsVues.put(m, v);		
		m.start();
		return m;
	}
	/**
	 * Créer un match entre plusieurs équipes :
	 * 
	 * @param nbrEquipes nombre d'équipes 
	 * @param nbrJoueurs nombre de joueurs par équipe
	 * @param noms tableau de noms des équipes
	 * @param colors tableau de couleur des équipes
	 * @return Match
	 */

	public Match initMatch(int nbrEquipes, int nbrJoueurs, String[] noms, int[]  colors) {
		Match m = new Match(nbrEquipes, nbrJoueurs, noms,colors);
		VueJeuEquipe v = new VueJeuEquipe(m);
		m.addObserver(v);
		m.addObserver(this.vue);
		this.modelsVues.put(m, v);		
		m.start();
		return m;
	}
	/**
	 * Termine un match. Permet aussi de détruire la Vue du match. 
	 * 
	 * @param m Match
	 * 
	 */
	public void endMatch(Match m){
		VueJeuEquipe v = (VueJeuEquipe) this.modelsVues.get(m);
		v.destroy();
		this.modelsVues.remove(m);
	}
	


	//
	// Accessor methods
	//



	//
	// Other methods
	//

}
