package pInclude;

import java.awt.Point;
import java.io.Serializable;

/**
 * A Vector2D represents a scalar value with a direction.  Vectors are immutable and hence
 * all manipulating operations return a new Vector.
 *
 * @author Matthew Carroll
 *
 */
public class Vector2D implements Serializable {

        public static final Vector2D ZERO = new Vector2D(new Point2D(0,0));
       
        private Point2D vPoint;
    private static final long serialVersionUID = 7122655165025442948L;

    public Vector2D(Point2D p) {
                vPoint = p;
        }
       
        public Vector2D(float mag, Angle theta) {
                float x = (float)(Math.cos(theta.getRadians()) * mag);
                float y = (float)(Math.sin(theta.getRadians()) * mag);
               
                vPoint = new Point2D(x, y);
        }
       
        public Vector2D(Point2D source, Point2D end) {          
                float x = end.getX() - source.getX();
                float y = end.getY() - source.getY();
               
                vPoint = new Point2D(x, y);
        }
        public Vector2D(Point source, Point end){
        	float x = (float) (end.getX() - source.getX());
            float y = (float) (end.getY() - source.getY());
           
            vPoint = new Point2D(x, y);
      	}
        public Vector2D(float x, float y){
        	vPoint = new Point2D(x, y);
    	}
        public Angle getAngle() {
                return new Angle((float)Math.atan2(vPoint.getY(), vPoint.getX()));
        }
       
        public Vector2D add(Vector2D v) {
                Vector2D result = new Vector2D(vPoint.add(v.vPoint));
               
                return result;
        }
       
        /*
         * Subtracts v from this vector
         */
        public Vector2D subtract(Vector2D v) {
                return new Vector2D(vPoint.subtract(v.vPoint));
        }
       
        public float getMagnitude() {
                return (float)Math.sqrt(Math.pow(vPoint.getX(), 2) + Math.pow(vPoint.getY(), 2));
        }
       
        public Point2D getVectorAsPoint2D() {
                return vPoint;
        }
       
        public Vector2D normalize() {
                float mag = getMagnitude();

                float normalX = vPoint.getX() / mag;
                float normalY = vPoint.getY() / mag;

                return new Vector2D(new Point2D(normalX, normalY));
        }
       
        public Vector2D scale(float scaleFactor) {
                return new Vector2D(getMagnitude() * scaleFactor, getAngle());
        }
       
        public boolean equals(Vector2D v) {
                if( (vPoint.getX() == v.vPoint.getX()) && (vPoint.getY() == v.vPoint.getY()) ) {
                        return true;
                }else{
                        return false;
                }
        }
       
        public String toString() {
                return "(" + vPoint.getX() + ", " + vPoint.getY() + ")";
        }
       
}
