/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pModel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import pInclude.Observable;
import pInclude.Observer;
import pModel.pEquipe.AbstractTactic;
import pModel.pEquipe.Attaque;
import pModel.pEquipe.Equipe;
import pModel.pTerrain.Terrain;
import pModel.pTerrain.ZoneTerrain;
import pModel.pTortue.Balle;
import pModel.pTortue.Tortue;
import pModel.pTortue.pCatacteristiques.Deplacement;
import pModel.pTortue.pCatacteristiques.Interception;

/**
 * Class Match
 * @author Olivier Berthier
 */
public class Match extends Thread implements _Model, Observable {

	//
	// Fields
	//
	/**
	 * Titre du match
	 */
	private String title;

	/**
	 * Liste des abonnées à Observer
	 */
	private ArrayList<Observer> listObserver = new ArrayList<Observer>();
	/**
	 * Terrain du match
	 */
	private Terrain terrain;
	/**
	 * Vitesse du match
	 */
	private int speed=30;
	/**
	 * Référence d'une instance de balle
	 */
	private Balle balle;
	/**
	 * @see pModel._Model#run()
	 */
	private boolean run;
	/**
	 * Met le match en pause quand Vrai
	 */
	private boolean pause;
	/**
	 * Liste des équipes du match
	 */
	private ArrayList<Equipe> equipes = new ArrayList<Equipe>();
	/**
	 * Liste des tortues qui jouent
	 */
	private ArrayList<Tortue> tortues = new ArrayList<Tortue>();



	//
	// Constructors
	//
	/**
	 * Constructeur de Match
	 */
	public Match () { 
		this.init();	  	  
	};

	public Match(int nbrEquipes, int nbrJoueurs, String[] noms,int[] colors) {
		this.init( nbrEquipes, nbrJoueurs, noms,colors);
	}



	//
	// Methods
	//
	/**
	 * Fait Jouer les tortues tant que run est vrai et si pause et faux
	 * 
	 * @see java.lang.Thread#run()
	 * @see pModel._Model#run()
	 */
	@Override
	public void run(  )
	{

		while(this.run){
			if(!this.pause){
				for (Iterator<Equipe> it = this.equipes.iterator();it.hasNext();) {
					Equipe e = (Equipe) it.next();
					e.getTactic().bouger(this.terrain, e.getMembres());
				}
				this.getBalle().setPosition();
				notifyObserver();
			}
			try {
				Thread.sleep(speed);
			} catch (InterruptedException e) {
				e.printStackTrace();
				Thread.currentThread().interrupt(); 
				break;
			}

		}	  
	}


	/**
	 * Initialise le match
	 * @see pModel._Model#init()
	 */
	@Override
	public void init(  )
	{
		int i,j,nbrEquipes=2;
		boolean colorOk;
		int color;
		this.run=true;
		this.pause=false;
		terrain = new Terrain(800,800);
		//définir une sone limite pour le terrain
		ZoneTerrain bordures = new ZoneTerrain((int)terrain.getMinX(),(int)terrain.getMinY(),(int)terrain.getHeight(),(int)terrain.getHeight(),3);
		//Créer une zone de 10% de moins que la zone principale, pour la bordure du terrain
		ZoneTerrain limites=bordures.reductionPct(10);
		limites.setTypeZone(2);
		terrain.addZones(bordures);
		terrain.addZones(limites);

		ArrayList<ZoneTerrain> zonesEquipes=limites.divisionNbrZones(nbrEquipes);

		terrain.getZonesList().addAll(zonesEquipes);


		for(i=0;i<nbrEquipes;i++){

			do{
				color = (int) (Math.random()*12);
				colorOk = true;
				for(Equipe e : this.equipes){
					if(e.getCouleurId()==color) colorOk=false;
				}
			}while(!colorOk);
			ZoneTerrain embut = zonesEquipes.get(i).addEmbut();
			terrain.getZonesList().add(embut);
			Equipe e = new Equipe(embut,zonesEquipes.get(i),color);	

			for(j=0;j<5;j++){
				Tortue t=null;
				if(j%3==0){
					//Créer des tortues rapides
					t = new Deplacement(this, (int)(Math.random() * (zonesEquipes.get(i).getWidth())+zonesEquipes.get(i).getMinX()),(int)(Math.random() * (zonesEquipes.get(i).getHeight())+zonesEquipes.get(i).getMinY()),(int)(Math.random() * 360));

				}else if(j%2==0){
					//Créer des tortues d'interception
					t = new Interception(this, (int)(Math.random() * (zonesEquipes.get(i).getWidth())+zonesEquipes.get(i).getMinX()),(int)(Math.random() * (zonesEquipes.get(i).getHeight())+zonesEquipes.get(i).getMinY()),(int)(Math.random() * 360));

				}else{
					//Créer des tortues de lancer
					t = new Deplacement(this, (int)(Math.random() * (zonesEquipes.get(i).getWidth())+zonesEquipes.get(i).getMinX()),(int)(Math.random() * (zonesEquipes.get(i).getHeight())+zonesEquipes.get(i).getMinY()),(int)(Math.random() * 360));

				}
				e.addMembre(t);
				tortues.add(t);
			}
			//AbstractTactic tactic = new Agressif(e); 
			AbstractTactic tactic = new Attaque(e);
			e.setTactic(tactic);

			this.addEquipes(e);
		}
		balle = new Balle(this,(int)terrain.getWidth()/2,(int)terrain.getHeight()/2);
		this.title="Tortue Equipes";
		Collections.shuffle(tortues);
		terrain.sortDec();
	}
	/**
	 * Initialise le match :
	 * 
	 * @param nbrEquipes nombre d'équipes
	 * @param nbrJoueurs nombre de joueurs par équipes
	 * @param noms tableau de nom des équipes
	 * @param colors tableau de couleurs des équipes
	 */
	private void init(int nbrEquipes, int nbrJoueurs, String[] noms,int[] colors) {

		int i,j;
		boolean colorOk;
		int color;
		this.run=true;
		this.pause=false;
		terrain = new Terrain(400,400);
		//définir une sone limite pour le terrain
		ZoneTerrain bordures = new ZoneTerrain((int)terrain.getMinX(),(int)terrain.getMinY(),(int)terrain.getHeight(),(int)terrain.getHeight(),3);
		//Créer une zone de 10% de moins que la zone principale, pour la bordure du terrain
		ZoneTerrain limites=bordures.reductionPct(10);
		limites.setTypeZone(2);
		terrain.addZones(bordures);
		terrain.addZones(limites);

		ArrayList<ZoneTerrain> zonesEquipes=limites.divisionNbrZones(nbrEquipes);

		terrain.getZonesList().addAll(zonesEquipes);


		for(i=0;i<nbrEquipes;i++){

			ZoneTerrain embut = zonesEquipes.get(i).addEmbut();
			terrain.getZonesList().add(embut);
			Equipe e = new Equipe(embut,zonesEquipes.get(i),colors[i]);	
			e.setNom(noms[i]);
			for(j=0;j<nbrJoueurs;j++){
				Tortue t=null;
				if(j%3==0){
					//Créer des tortues rapides
					t = new Deplacement(this, (int)(Math.random() * (zonesEquipes.get(i).getWidth())+zonesEquipes.get(i).getMinX()),(int)(Math.random() * (zonesEquipes.get(i).getHeight())+zonesEquipes.get(i).getMinY()),(int)(Math.random() * 360));

				}else if(j%2==0){
					//Créer des tortues d'interception
					t = new Interception(this, (int)(Math.random() * (zonesEquipes.get(i).getWidth())+zonesEquipes.get(i).getMinX()),(int)(Math.random() * (zonesEquipes.get(i).getHeight())+zonesEquipes.get(i).getMinY()),(int)(Math.random() * 360));

				}else{
					//Créer des tortues de lancer
					t = new Deplacement(this, (int)(Math.random() * (zonesEquipes.get(i).getWidth())+zonesEquipes.get(i).getMinX()),(int)(Math.random() * (zonesEquipes.get(i).getHeight())+zonesEquipes.get(i).getMinY()),(int)(Math.random() * 360));

				}
				e.addMembre(t);
				tortues.add(t);
			}
			//AbstractTactic tactic = new Agressif(e); 
			AbstractTactic tactic = new Attaque(e);
			e.setTactic(tactic);

			this.addEquipes(e);
		}
		balle = new Balle(this,(int)terrain.getWidth()/2,(int)terrain.getHeight()/2);
		this.title="Tortue Equipes";
		Collections.shuffle(tortues);
		terrain.sortDec();

	}
	/**
	 * Méthode permettznt d'alterner Pause
	 * 
	 * @see pModel.Match#pause
	 */
	public void toggleRun(){
		this.pause =! pause;
	}


	/**
	 * Méthode permettant d'arrêter un match
	 * @see pModel._Model#end()
	 * @see pModel.Match#run
	 */
	@Override
	public void end(  ){
		this.run=false;
	}
	//
	// Accessor methods
	//
	/**
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @param title the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}
	/**
	 * Set the value of plateau
	 * @param newVar the new value of plateau
	 */
	private void setTerrain ( Terrain newVar ) {
		terrain = newVar;
	}

	/**
	 * Get the value of plateau
	 * @return the value of plateau
	 */
	public Terrain getTerrain ( ) {
		return terrain;
	}
	/**
	 * Set the value of balle
	 * @param newVar the new value of balle
	 */
	private void setBalle ( Balle newVar ) {
		balle = newVar;
	}

	/**
	 * Get the value of balle
	 * @return the value of balle
	 */
	public Balle getBalle ( ) {
		return balle;
	}

	/**
	 * Get the List of Equipes objects held by equipesVector
	 * @return List of Equipes objects held by equipesVector
	 */
	public ArrayList getEquipes ( ) {
		return equipes;
	}


	/**
	 * Add a Equipes object to the equipesVector List
	 */
	public void addEquipes ( Equipe new_object ) {
		equipes.add(new_object);
	}

	/**
	 * Remove a Equipe object from equipes ArrayList
	 */
	public void removeEquipes ( Equipe new_object )
	{
		equipes.remove(new_object);
	}



	/**
	 * @return the tortues
	 */
	public ArrayList<Tortue> getTortues() {
		return tortues;
	}

	/**
	 * @param tortues the tortues to set
	 */
	public void setTortues(ArrayList<Tortue> tortues) {
		this.tortues = tortues;
	}








	/**
	 * @return the speed
	 */
	public int getSpeed() {
		return speed;
	}

	/**
	 * @param speed the speed to set
	 */
	public void setSpeed(int speed) {
		this.speed = speed;
	}  


	//
	// Other methods
	//


	/**
	 * 
	 * @param vue
	 */
	/*public void addObserver(_Vue vue) {
		this.listObserver.add((Observer) vue);
	}*/

	/* (non-Javadoc)
	 * @see pInclude.Observable#notifyObserver()
	 */
	@Override
	public void notifyObserver() {
		for(Observer obs : listObserver)
			obs.update();
	}

	/* (non-Javadoc)
	 * @see pInclude.Observable#removeObserver()
	 */
	@Override
	public void removeObserver() {
		listObserver = new ArrayList<Observer>();
	}

	/* (non-Javadoc)
	 * @see pInclude.Observable#addObserver(pInclude.Observer)
	 */
	@Override
	public void addObserver(Observer obs) {
		this.listObserver.add(obs);

	}

}
