/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pModel.pEquipe;

import java.util.ArrayList;
import java.util.Iterator;

import javax.management.relation.Role;

import pInclude.Observer;
import pModel.pRole.AbstractRole;
import pModel.pTerrain.Terrain;
import pModel.pTortue.Tortue;

/**
 * Class AbstractTactic
 * @author Olivier Berthier
 */
abstract public class AbstractTactic implements Observer {

	//
	// Fields
	//

	/**
	 * Equipe associée à l'instance de la tactic.
	 * @see Equipe
	 */
	private Equipe equipe;

	//
	// Constructors
	//
	/**
	 * Constructeur par défaut
	 */
	public AbstractTactic () { };
	/**
	 * Constructeur 
	 * @param equipe
	 * 
	 * @see Equipe
	 */
	public AbstractTactic (Equipe equipe) {
		this.equipe=equipe;
	};
	//
	// Methods
	//


	//
	// Accessor methods
	//

	/**
	 * Set the value of m_tactic
	 * @param newVar the new value of m_tactic
	 */
	public void setEquipe ( Equipe newVar ) {
		equipe = newVar;
	}

	/**
	 * Get the value of m_tactic
	 * @return the value of m_tactic
	 */
	public Equipe getEquipe ( ) {
		return equipe;
	}

	//
	// Other methods
	//

	/**
	 * Actualiser les Roles des joueurs de l'équipe.
	 * 
	 * @see Role
	 */
	abstract public void actualiserTactic(  );


	/**
	 * TODO
	 */
	public void positionnerTortues(  )
	{
	}


	/**
	 * TODO
	 */
	public void trierTortues(  )
	{
	}


	/**
	 * Méthode permettant d'appliquer les des roles associés aux tortues
	 * 
	 * @param terrain
	 * @param tortues
	 * 
	 * @see AbstractRole#gestionDeplacements(Terrain)
	 * @see AbstractRole#gestionBalle()
	 * @see Terrain
	 * @see Tortues
	 */
	public final void bouger( Terrain terrain, ArrayList<Tortue> tortues ){
		for (Iterator<Tortue> it = tortues.iterator();it.hasNext();) {
			Tortue t = (Tortue) it.next();			
			t.getRole().gestionDeplacements(terrain);
			t.getRole().gestionBalle();
		}
	}

	/* (non-Javadoc)
	 * @see pInclude.Observer#update()
	 */
	@Override
	public void update() {
		// TODO Auto-generated method stub
		//Analyser les positions des tortues et de la balle pour déterminer la tactic à prendre
	}
}
