/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pModel.pEquipe;

import java.awt.Color;
import java.util.ArrayList;

import pModel.pTerrain._Zone;
import pModel.pTortue.Tortue;


/**
 * Class Equipe
 * Classe permettant de créer une équipe de tortue 
 * @author Olivier Berthier
 */
public class Equipe   {

	//
	// Fields
	//

	/**
	 * Nom de l'équipe 
	 */
	private String nom;
	/**
	 * Score de l'équipe
	 */
	private int score;
	/**
	 * Membres de l'équipe
	 */
	private ArrayList<Tortue> membres = new ArrayList<Tortue>();
	/**
	 * Tactic de l'équipe
	 */
	private AbstractTactic tactic;
	/**
	 * Couleur des joueurs de l'équipe
	 */
	private int couleur;
	/**
	 * Zone associée à l'équipe
	 */
	private _Zone zone;
	/**
	 * Embut de l'équipe
	 */
	private _Zone enbut;
	//
	// Constructors
	//
	/**
	 * Constructeur de la classe Equipe.
	 * 
	 * @param enbut
	 */
	public Equipe (_Zone embut ) { 
		this.couleur=(int)(Math.random() * (11));
		this.enbut=embut;
	};

	/**
	 * Constructeur de la classe Equipe.
	 * 
	 * @param enbut
	 * @param couleur
	 */
	public Equipe (_Zone embut,int couleur) { 
		this.couleur=couleur;
		this.enbut=embut;
	};
	/**
	 * Constructeur de la classe Equipe.
	 * 
	 * @param enbut
	 * @param zone
	 */
	public Equipe (_Zone embut,_Zone zone) {
		this.setZone(zone);	
		this.couleur=(int)(Math.random() * (11));
		this.enbut=embut;
	};
	/**
	 * Constructeur de la classe Equipe.
	 * 
	 * @param enbut
	 * @param zone
	 * @param couleur
	 */
	public Equipe (_Zone embut,_Zone zone,int couleur) {
		this.setZone(zone);
		this.couleur=couleur;
		this.enbut=embut;
	};
	//
	// Methods
	//

	/**
	 * Retourne le nombre de joueurs de l'équipe.
	 * 
	 * @return       int
	 */
	public int nbrJoueurs(  )
	{
		return membres.size();
	}


	/**
	 * Incrémente le score de l'équipe de 1 
	 */
	public void incrScore(  )
	{		
		this.score++;
	}


	/**
	 * Réinitialise le score de l'équipe
	 */
	public void resetScore(  )
	{
		this.score=0;
	}
	
	//
	// Accessor methods
	//

	/**
	 * Set the value of nom
	 * @param newVar the new value of nom
	 */
	public void setNom ( String newVar ) {
		nom = newVar;
	}

	/**
	 * Get the value of nom
	 * @return the value of nom
	 */
	public String getNom ( ) {
		return nom;
	}

	/**
	 * Set the value of score
	 * @param newVar the new value of score
	 */
	@SuppressWarnings("unused")
	private void setScore ( int newVar ) {
		score = newVar;
	}

	/**
	 * Get the value of score
	 * @return the value of score
	 */
	public int getScore ( ) {
		return score;
	}

	/**
	 * Set the value of membres
	 * @param newVar the new value of membres
	 */
	public void setMembres ( ArrayList<Tortue> newVar ) {
		membres = newVar;
	}

	/**
	 * Get the value of membres
	 * @return the value of membres
	 */
	public ArrayList<Tortue> getMembres ( ) {
		return membres;
	}

	/**
	 * Set the value of tactic
	 * @param newVar the new value of tactic
	 */
	public void setTactic ( pModel.pEquipe.AbstractTactic newVar ) {
		this.tactic = newVar;
		this.tactic.actualiserTactic();
	}

	/**
	 * Get the value of tactic
	 * @return the value of tactic
	 */
	public AbstractTactic getTactic ( ) {
		return tactic;
	}

	/**
	 * Add a Membre object to the membreVector List
	 */
	public void addMembre ( Tortue new_object ) {
		new_object.setCouleur(couleur);
		new_object.setEquipe(this);
		membres.add(new_object);
	}

	/**
	 * Remove a Membre object from membreVector List
	 */
	public void removeMembre ( Tortue new_object )
	{
		membres.remove(new_object);
	}



	/**
	 * Retourne le code de la couleur de l'équipe
	 * @return int
	 */
	public int getCouleurId() {		
		return couleur;
	}
	/**
	 * Retourne la couleur de l'équipe
	 * @return Color
	 * 
	 * @see Color
	 */
	public Color getCouleur ( ) {
		switch(couleur) {
		case 0: return(Color.black);
		case 1: return(Color.blue);
		case 2: return(Color.cyan);
		case 3: return(Color.darkGray);
		case 4: return(Color.red);
		case 5: return(Color.green);
		case 6: return(Color.lightGray);
		case 7: return(Color.magenta);
		case 8: return(Color.orange);
		case 9: return(Color.gray);
		case 10: return(Color.pink);
		case 11: return(Color.yellow);
		default : return(Color.black);
		}
	}

	/**
	 * Défini la couleur de l'équipe
	 * @param couleur the couleur to set
	 */
	public void setCouleur(int couleur) {
		this.couleur = couleur;
	}
	/**
	 * Retourne la zone de l'équipe
	 * @return the zone
	 */
	public _Zone getZone() {
		return zone;
	}

	/**
	 * Défini la zone de l'équipe
	 * @param zone the zone to set
	 */
	public void setZone(_Zone zone) {
		this.zone = zone;
		this.zone.setTypeZone(1);
	}
	
	/**
	 * Retourne l'enbut de l'équipe
	 * @return the enbut
	 */
	public _Zone getEmbut() {
		return enbut;
	}

	/**
	 * Défini l'enbut de l'équipe
	 * @param enbut the enbut to set
	 */
	public void setEmbut(_Zone embut) {
		this.enbut = embut;
	}
	//
	// Other methods
	//

	


	

	


}
