/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pModel.pTerrain;

import java.awt.Point;
import java.awt.Polygon;

import pInclude.Vector2D;
import pModel.pTortue.AbstractTortue;
/**
 * Class ZoneJeu 
 * Permet de définir des zones polygonnales sur le terrain.
 * @author Olivier Berthier
 */
public class ZoneJeu extends Polygon implements Comparable,_Zone  {



	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//
	// Fields
	//
	//
	// Constructors
	//
	/**
	 * 1=> equipe
	 * 0=> enbut
	 * 2=> limite 
	 * 3=> sortie
	 */
	private int typeZone; 
	
	/**
	 * Constructeur par défaut 
	 */
	public ZoneJeu () {
		super();
	};
	/**
	 * Constructeur prennant un type de zone en argument
	 * @param typeZone int
	 */
	public ZoneJeu (int typeZone) {
		super();
		this.setTypeZone(typeZone);
	};

	/**
	 * Retourne une ZoneJeu réduction de pct pourcents de this.
	 * 
	 * @param pct float
	 * @return ZoneJeu
	 */
	public ZoneJeu reductionPct(float pct){
		//Créer une zone réduction de pct% de la zone z
		Point g = this.centreGavite();
		//pour chaque segment (point,centre de gravité), déterminer le point à pct% du module depuis le point
		Vector2D v,v2;
		ZoneJeu result=new ZoneJeu();
		for(int i=0;i<this.npoints-1;i++){
			v=new Vector2D(g,new Point(this.xpoints[i],this.ypoints[i]));
			v2=new Vector2D(v.getVectorAsPoint2D().getX()*(1-(pct/100)),v.getVectorAsPoint2D().getY()*(1-(pct/100)));
			result.addPoint((int)(g.getX()+v2.getVectorAsPoint2D().getX()), (int)(g.getY()+v2.getVectorAsPoint2D().getY()));
		}

		return result;
	}
	/* (non-Javadoc)
	 * @see pModel.pTerrain._Zone#centreGavite()
	 */
	@Override
	public Point centreGavite(){	  
		double xG=0, yG=0, tmp, aire=this.surface();
		for(int i=0;i<this.npoints-1;i++){
			tmp= (this.xpoints[i]*this.ypoints[i+1] - this.xpoints[i+1]*this.ypoints[i]);
			xG += tmp * (this.xpoints[i]+this.xpoints[i+1]);
			yG += tmp * (this.ypoints[i]+this.ypoints[i+1]);
		}
		aire/=2;
		xG=(xG/(6*aire));
		yG=(yG/(6*aire));
		Point p =new Point();
		p.setLocation(xG, yG);
		return p;
	}

	/* (non-Javadoc)
	 * @see pModel.pTerrain._Zone#surface()
	 */
	@Override
	public double surface(){
		double result=0;
		for(int i=0;i<this.npoints-1;i++){
			result += (this.xpoints[i]*this.ypoints[i+1] - this.xpoints[i+1]*this.ypoints[i]);
		}
		return result;
	}


	/* (non-Javadoc)
	 * @see pModel.pTerrain._Zone#contains(pModel.pTortue.AbstractTortue)
	 */
	@Override
	public boolean contains(AbstractTortue t){
		return this.contains(t.getX(), t.getY());
	}
	//
	// Accessor methods
	//
	/* (non-Javadoc)
	 * @see pModel.pTerrain._Zone#getTypeZone()
	 */
	@Override
	public int getTypeZone() {
		return typeZone;
	}
	/* (non-Javadoc)
	 * @see pModel.pTerrain._Zone#setTypeZone(int)
	 */
	@Override
	public void setTypeZone(int typeZone) {
		this.typeZone = typeZone;
	}

	//
	// Other methods
	//
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString(){
		return ""+this.npoints;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(Object o) {
		ZoneJeu z =(ZoneJeu) o;
		if(this.surface()>=z.surface())
			return 1;
		return 0;
	}
}

