/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pModel.pTortue;

import java.awt.Color;
import java.awt.Point;

import pInclude.Vector2D;
import pModel._Model;
import pModel.pTerrain._Zone;


/**
 * Class AbstractTortue
 * 
 * @author Olivier Berthier
 */
abstract public class AbstractTortue {

	//
	// Fields
	//

	protected static final double ratioDegRad = 0.0174533; // Rapport radians/degres (pour la conversion)

	/**
	 * Abscisse de l'instance d'AbstractTortue 
	 */
	protected int x;
	/**
	 * Ordonnée  de l'instance d'AbstractTortue
	 */
	protected int y;
	/**
	 * Direction de l'instance d'AbstractTortue (angle en °)
	 */
	protected int direction;
	/**
	 * Couleur de l'instance d'AbstractTortue
	 */
	protected int couleur;
	/**
	 * Rayon d'action de la tortue
	 */
	protected int rayon = 10;

	/**
	 * Jeu dans lequel la tortue est instanciée
	 */
	protected _Model model;
	//
	// Constructors
	//
	/**
	 * Constructeur par défaut
	 */
	public AbstractTortue() {
		// TODO Auto-generated constructor stub
	}
	/**
	 * Constructeur AbstractTortue :
	 * 
	 * @param model Jeu dans lequel l'AbstractTortue est instanciée
	 * 
	 */
	public AbstractTortue (_Model model) { 
		this.x=0;
		this.y=0;
		this.direction=0;
		this.couleur=4;
		this.rayon=15;
		this.model=model;

	};

	/**
	 * Constructeur AbstractTortue :
	 * 
	 * @param model Jeu dans lequel l'AbstractTortue est instanciée
	 * @param x Abscisse de l'AbstractTortue
	 * @param y Ordonnée de l'AbstractTortue
	 */
	public AbstractTortue (_Model model,int x, int y) {
		this.x=x;
		this.y=y;
		this.direction=0;
		this.couleur=4;
		this.rayon=15;
		this.model=model;
	};
	/**
	 * Constructeur AbstractTortue :
	 * 
	 * @param model Jeu dans lequel l'AbstractTortue est instanciée
	 * @param x Abscisse de l'AbstractTortue
	 * @param y Ordonnée de l'AbstractTortue
	 * @param dir Direction de la tortue en degrés
	 */
	public AbstractTortue (_Model model,int x, int y , int dir) {
		this.x=x;
		this.y=y;
		this.direction=dir;
		this.couleur=4;
		this.rayon=15;
		this.model=model;
	};
	//
	// Methods
	//


	//
	// Accessor methods
	//


	/**
	 * Set the value of x
	 * @param newVar the new value of x
	 */
	protected void setX ( int newVar ) {
		x = newVar;
	}

	/**
	 * Get the value of x
	 * @return the value of x
	 */
	public int getX ( ) {
		return x;
	}

	/**
	 * Set the value of y
	 * @param newVar the new value of y
	 */
	protected void setY ( int newVar ) {
		y = newVar;
	}

	/**
	 * Get the value of y
	 * @return the value of y
	 */
	public int getY ( ) {
		return y;
	}

	/**
	 * Set the value of direction
	 * @param newVar the new value of direction
	 */
	protected void setDirection ( int newVar ) {
		direction = newVar;
	}

	/**
	 * Get the value of direction
	 * @return the value of direction
	 */
	public int getDirection ( ) {
		return direction;
	}

	/**
	 * Set the value of couleur
	 * @param newVar the new value of couleur
	 */
	public void setCouleur ( int newVar ) {
		couleur = newVar;
	}

	/**
	 * Get the value of couleur
	 * @return the value of couleur
	 */
	public Color getCouleur ( ) {
		switch(couleur) {
		case 0: return(Color.black);
		case 1: return(Color.blue);
		case 2: return(Color.cyan);
		case 3: return(Color.darkGray);
		case 4: return(Color.red);
		case 5: return(Color.green);
		case 6: return(Color.lightGray);
		case 7: return(Color.magenta);
		case 8: return(Color.orange);
		case 9: return(Color.gray);
		case 10: return(Color.pink);
		case 11: return(Color.yellow);
		default : return(Color.black);
		}
	}

	/**
	 * Set the value of rayon
	 * @param newVar the new value of rayon
	 */
	protected void setRayon ( int newVar ) {
		rayon = newVar;
	}

	/**
	 * Get the value of rayon
	 * @return the value of rayon
	 */
	public int getRayon ( ) {
		return rayon;
	}

	/**
	 * Retourne le Jeu de l'AbsractTortue
	 * @return
	 */
	public _Model getModel() {
		return model;
	}

	/**
	 * Défini le Jeu de l'AbsractTortue
	 * 
	 * @param model
	 */
	public void setModel(_Model model) {
		this.model = model;
	}
	//
	// Other methods
	//



	/**
	 * Fait avancer l'AbsractTortue
	 * @param        distance
	 */
	public void avancer( int distance )
	{	
		x = (int) Math.round(x+distance*Math.cos(ratioDegRad*direction));
		y = (int) Math.round(y+distance*Math.sin(ratioDegRad*direction));
	}


	/**
	 * Fait tourner l'AbsractTortue à droite
	 * @param        angle
	 */
	public void droite( int angle )
	{
		direction = (direction + angle) % 360;
	}


	/**
	 * Fait tourner l'AbsractTortue à gauche
	 * 
	 * @param        angle
	 */
	public void gauche( int angle )
	{
		direction = (direction - angle) % 360;
	}


	/**
	 * Réinitialise l'AbsractTortue.
	 * 
	 * //TODO :> au centre du terrain
	 */
	public void reset(  )
	{
		this.x=40;
		this.y=40;
		this.direction=0;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString(){
		return "("+this.getX()+";"+this.getY()+")";
	}

	/**
	 * Retourne la distance entre l'AbstractTortue et une autre tortue.
	 * 
	 * @param t Tortue
	 * @return distance
	 */
	public int distance(AbstractTortue t){		
		return  (int) new Point(this.x,this.y).distance(t.getX(), t.getY());
	}

	/**
	 * Dirige l'AbstractTortue vers le point P
	 * 
	 * @param p Point
	 */
	public void deplacementVers( Point p )
	{
		Vector2D test = new Vector2D(new Point(this.getX(),this.getY()), p);	  
		this.setDirection((int)test.getAngle().getDegrees());

	}
	
	/**
	 * Fait tourner la tortue en direction de la tortue passée en paramètre
	 * 
	 * @param        t
	 */
	public void deplacementVers( pModel.pTortue.AbstractTortue t )
	{
		Vector2D test = new Vector2D(new Point(this.getX(),this.getY()), new Point(t.getX(),t.getY()));	  
		this.setDirection((int)test.getAngle().getDegrees());

	}
	/**
	 * Vrai si la l'AbstractTortue est dans la zone, sinon faux
	 * @param z Zone
	 * @return boolean
	 */
	public boolean collision(_Zone z){
		if(z.contains(this)) return false;
		else return true;
	}


}
