/**
 *
 * Copyright (C) 2012 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

package pModel.pTortue;

import java.util.ArrayList;
import java.util.Iterator;

import pInclude.Observable;
import pInclude.Observer;
import pModel.Match;
import pModel._Model;
import pModel.pEquipe.Equipe;
import pModel.pRole.AbstractRole;

/**
 * Class Tortue
 * Permet de créer des Tortues
 *
 * @author Olivier Berthier
 *
 */
public class Tortue extends AbstractTortue implements Observable {

	//
	// Fields
	//

	/**
	 * Nom de la tortue
	 */
	private String nom;

	/**
	 * Equipe de la tortue
	 */
	private Equipe equipe;

	/**
	 * Rôle de la tortue
	 */
	private AbstractRole role;
	//
	// Constructors
	//
	/**
	 * Constructeur pas défaut
	 */
	public Tortue() {
		super();
	}
	/**
	 * Constructeur de Tortue :
	 * 
	 * @param model Jeu dans lequel la Tortue est instanciée
	 */
	public Tortue (_Model model) {
		super(model);	  
	};

	/**
	 * Constructeur  de Tortue :
	 * 
	 * @param model jeu dans lequel la Tortue est instanciée
	 * @param x Abscisse de la Tortue
	 * @param y Ordonnée de la Tortue
	 */
	public Tortue (_Model model,int x, int y) {
		super(model,x,y);
	};
	/**
	 * Constructeur de Tortue :
	 * 
	 * @param model Jeu dans lequel la Tortue est instanciée
	 * @param x Abscisse de la Tortue
	 * @param y Ordonnée de la Tortue
	 * @param dir Direction de la Tortue
	 */
	public Tortue ( _Model model,int x, int y , int dir) {
		super(model,x,y,dir);
	}
	/**
	 * Constructeur de Tortue :
	 * 
	 * @param model Jeu dans lequel la Tortue est instanciée
	 * @param nom Nom de la Tortue
	 */
	public Tortue (_Model model,String nom) { 
		super(model);
		this.nom=nom;
	};

	/**
	 * Constructeur de Tortue :
	 * 
	 * @param model Jeu dans lequel la Tortue est instanciée
	 * @param nom Nom de la Tortue
	 * @param x Abscisse de la Tortue
	 * @param y Ordonnée de la Tortue
	 */
	public Tortue (_Model model,String nom, int x, int y) {
		super(model,x,y);
		this.nom=nom;
	};
	/**
	 * Constructeur de Tortue :
	 * 
	 * @param model  Jeu dans lequel la Tortue est instanciée
	 * @param nom Nom de la Tortue
	 * @param x Abscisse de la Tortue
	 * @param y Ordonnée de la Tortue
	 * @param dir Direction de la Tortue
	 */
	public Tortue (_Model model,String nom, int x, int y , int dir) {
		super(model,x,y,dir);
		this.nom=nom;
	}
	//
	// Methods
	//

	/* (non-Javadoc)
	 * @see pModel.pTortue.AbstractTortue#avancer(int)
	 */
	@Override
	public void avancer( int distance )
	{	
		this.gestionCollisions();
		x = (int) Math.round(x+distance*Math.cos(ratioDegRad*direction));
		y = (int) Math.round(y+distance*Math.sin(ratioDegRad*direction));
		//this.gestionBalle();

	}


	/**
	 * Retourne la liste des instances de tortues en collision avec celle-ci
	 * @param tortues
	 * @return
	 */
	public ArrayList<Tortue> collisions(ArrayList<Tortue> tortues){
		ArrayList<Tortue> result = new ArrayList<Tortue>(); 
		for (Iterator<Tortue> it = tortues.iterator();it.hasNext();) {
			Tortue t = (Tortue) it.next();
			if(this.distance(t)<=this.rayon){
				result.add(t);
			}
		}
		return result;
	}


	/* (non-Javadoc)
	 * @see pModel.pTortue.AbstractTortue#reset()
	 */
	@Override
	public void reset(  )
	{
		this.setX((int)this.getEquipe().getZone().centreGavite().getX());
		this.setY((int)this.getEquipe().getZone().centreGavite().getY());
		this.direction=(int) (Math.random() * 360);
	}

	/**
	 * Méthode de gestion des collisions
	 */
	public void gestionCollisions(){
		Match match= (Match) this.getModel();
		//on modifie la trajectoire des deux tortues
		for (Iterator<Tortue> it = match.getTortues().iterator();it.hasNext();) {
			Tortue t = (Tortue) it.next();
			if(t!=this){
				if(this.distance(t)<=this.rayon){
					this.droite(90+(int)(Math.random()*180));
					t.droite(90+(int)(Math.random()*180));

				}
			}
		}
	}

	/**
	 * @deprecated
	 */
	public void gestionBalle(){

	}



	/**
	 * Trace un carré
	 * @param        d
	 * @deprecated
	 */
	public void carre( int d )
	{
	}


	/**
	 * @deprecated
	 */
	public void actualiserTactic(  )
	{
	}


	/* (non-Javadoc)
	 * @see pModel.pTortue.AbstractTortue#toString()
	 */
	@Override
	public String toString(){
		return this.getNom()+" : "+super.toString();
	}

	/* (non-Javadoc)
	 * @see pInclude.Observable#addObserver(pInclude.Observer)
	 */
	@Override
	public void addObserver(Observer obs) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see pInclude.Observable#removeObserver()
	 */
	@Override
	public void removeObserver() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see pInclude.Observable#notifyObserver()
	 */
	@Override
	public void notifyObserver() {
		// TODO Auto-generated method stub
		//mettre à jour la stratégie si la tortue a changé de Zone de jeu

	}


	/**
	 * Méthode de lancé de la balle
	 * @param t
	 */
	public void lancer( pModel.pTortue.AbstractTortue t )
	{

		if(Math.random()*10>9.8){
			model.getBalle().setTortue((Tortue) t);
		}

	}


	/**
	 * Méthode d'insterception de la balle
	 * @param        t
	 */
	public void intercepter( pModel.pTortue.AbstractTortue t )
	{
		if(t!=this.model.getBalle()){
			if(Math.random()*10>9.9){
				model.getBalle().setTortue(this);
			}
		}else model.getBalle().setTortue((Tortue) t);
	}

	//
	// Accessor methods
	//

	/**
	 * Défini le nom de la tortue
	 * @param newVar the new value of nom
	 */
	public void setNom ( String newVar ) {
		nom = newVar;
	}

	/**
	 * Retourne la nom de la tortue
	 * @return the value of nom
	 */
	public String getNom ( ) {
		return nom;
	}

	/**
	 * Défini l'équipe  de la tortue
	 * @param newVar the new value of equipe
	 */
	public void setEquipe ( Equipe newVar ) {
		equipe = newVar;
	}

	/**
	 * Retourne l'équipe de la tortue
	 * @return the value of equipe
	 */
	public Equipe getEquipe ( ) {
		return equipe;
	}

	//
	// Other methods
	//













	/**
	 * @return the role
	 */
	public AbstractRole getRole() {
		return role;
	}

	/**
	 * @param role the role to set
	 */
	public void setRole(AbstractRole role) {
		this.role = role;
	}




}
